# from sqlalchemy import create_engine, event
# from sqlalchemy.orm import Session
# from sqlalchemy.exc import OperationalError
# import json
# import pytest
# import logging

# # Required if this file runs before another that requires AWS mocking
# from moto import mock_aws  # noqa

# from mind_castle.stores import stores as stores_cache
# from models import Base, SimpleMemoryModel

# logger = logging.getLogger(__name__)
# logger.level = logging.DEBUG


# # Create an in-memory SQLite database
# engine = create_engine("sqlite://", echo=True)
# Base.metadata.create_all(engine)


# # Simulate a DB-offline error on commit
# def simulate_db_offline(session, flush_context):
#     raise OperationalError("Simulated DB offline", params=None, orig=None)


# def test_failed_commit_create():
#     """
#     Tests that if there is an error on DB commit after creating a secret,
#     that the new secret is deleted
#     """

#     session = Session(engine)
#     event.listen(session, "after_flush", simulate_db_offline)
#     test_object = SimpleMemoryModel(data="A very secret string")

#     with pytest.raises(OperationalError):
#         with session.begin():
#             session.add(test_object)
#             # commit is attempted when exiting this block,
#             # and on failure it’s rolled back immediately

#     # at this point the mind-castle after_rollback hook has already run
#     assert stores_cache["memory"].secrets == {}


# def test_failed_commit_update():
#     """
#     Tests that if there is an error on DB commit after updating a secret,
#     that the secret value is still the old value
#     """

#     session = Session(engine, autobegin=False)
#     test_object = SimpleMemoryModel(data="A very secret string")
#     with session.begin():
#         session.add(test_object)
#     logger.debug(f"Added secret: {stores_cache['memory'].secrets}")

#     assert json.dumps("A very secret string") in list(
#         stores_cache["memory"].secrets.values()
#     )

#     event.listen(session, "after_flush", simulate_db_offline)

#     with pytest.raises(OperationalError):
#         with session.begin():
#             test_object = session.get(SimpleMemoryModel, test_object.id)
#             test_object.data = "An updated secret string"
#             # commit is attempted when exiting this block,
#             # and on failure it’s rolled back immediately
#     logger.debug(stores_cache["memory"].secrets)
#     # at this point the mind-castle after_rollback hook has already run
#     assert json.dumps("A very secret string") in list(
#         stores_cache["memory"].secrets.values()
#     )
#     assert json.dumps("An updated secret string") not in list(
#         stores_cache["memory"].secrets.values()
#     )


# def test_failed_commit_delete():
#     """
#     Tests that if there is an error on DB commit after deleting a secret,
#     that the secret still exists
#     """

#     session = Session(engine, autobegin=False)
#     test_object = SimpleMemoryModel(data="A very secret string")
#     with session.begin():
#         session.add(test_object)
#     logger.debug(f"Added secret: {stores_cache['memory'].secrets}")

#     event.listen(session, "after_flush", simulate_db_offline)

#     assert json.dumps("A very secret string") in list(
#         stores_cache["memory"].secrets.values()
#     )

#     with pytest.raises(OperationalError):
#         with session.begin():
#             test_object = session.get(SimpleMemoryModel, test_object.id)
#             session.delete(test_object)
#             # commit is attempted when exiting this block,
#             # and on failure it’s rolled back immediately

#     # at this point the mind-castle after_rollback hook has already run
#     assert json.dumps("A very secret string") in list(
#         stores_cache["memory"].secrets.values()
#     )



import uuid
import os
from abc import ABC, abstractmethod
from pydantic import BaseModel, Field


class MindCastleSecret(BaseModel):
    key: uuid.UUID = Field(default_factory=uuid.uuid1)
    mind_castle_secret_type: str
    encrypted_value: str
    metadata: dict = Field(default_factory=dict)


class SecretStoreBase(ABC):
    """This is an abstract class that defines the interface for a secret store."""

    store_type = "base"
    required_config = [[]]
    optional_config = []
    key_prefix = os.environ.get("MIND_CASTLE_SECRET_KEY_PREFIX", "")

    @abstractmethod
    def create_secret(self, value: str) -> MindCastleSecret:
        raise NotImplementedError()

    @abstractmethod
    def retrieve_secret(self, secret: MindCastleSecret) -> str:
        raise NotImplementedError()

    @abstractmethod
    def update_secret(self, secret: MindCastleSecret, value: str) -> MindCastleSecret:
        raise NotImplementedError()

    @abstractmethod
    def delete_secret(self, secret: MindCastleSecret) -> None:
        raise NotImplementedError()



# from moto import mock_aws
# import pytest

# from mind_castle.stores.aws import AWSSecretsManagerSecretStore
# from mind_castle.exceptions import RetrieveSecretException


# @mock_aws
# def test_create_secret():
#     secret_store = AWSSecretsManagerSecretStore()
#     key = secret_store.get_secret_key()
#     response = secret_store.create_secret(key, "some_secret_value")

#     # Read the secret from AWS directly to check
#     client = secret_store.client  # Use the same auth etc as the store
#     boto_response = client.get_secret_value(SecretId=response["key"])
#     assert boto_response["SecretString"] == "some_secret_value"
#     assert response["mind_castle_secret_type"] == secret_store.store_type


# @mock_aws
# def test_retrieve_secret():
#     secret_store = AWSSecretsManagerSecretStore()
#     # Add secret directly to AWS
#     client = secret_store.client  # Use the same auth etc as the store
#     client.create_secret(Name="some_secret_key", SecretString="some_secret_value")

#     assert (
#         secret_store.retrieve_secret(
#             {"mind_castle_secret_type": "awssecretsstore", "key": "some_secret_key"}
#         )
#         == "some_secret_value"
#     )


# @mock_aws
# def test_delete_secret():
#     secret_store = AWSSecretsManagerSecretStore()
#     # Add secret directly to AWS
#     client = secret_store.client
#     client.delete_secret(SecretId="some_secret_key", ForceDeleteWithoutRecovery=True)
#     client.create_secret(Name="some_secret_key", SecretString="some_secret_value")

#     secret_store.delete_secret(
#         {"mind_castle_secret_type": "awssecretsstore", "key": "some_secret_key"}
#     )

#     with pytest.raises(RetrieveSecretException):
#         secret_store.retrieve_secret(
#             {"mind_castle_secret_type": "awssecretsstore", "key": "some_secret_key"}
#         )


# @mock_aws
# def test_update_secret():
#     secret_store = AWSSecretsManagerSecretStore()
#     # Add secret directly to AWS
#     client = secret_store.client
#     client.delete_secret(SecretId="some_secret_key", ForceDeleteWithoutRecovery=True)
#     client.create_secret(Name="some_secret_key", SecretString="some_secret_value")

#     secret_store.update_secret(
#         {"mind_castle_secret_type": "awssecretsstore", "key": "some_secret_key"},
#         "some_updated_value",
#     )

#     newvalue = secret_store.retrieve_secret(
#         {"mind_castle_secret_type": "awssecretsstore", "key": "some_secret_key"}
#     )
#     assert newvalue == "some_updated_value"



from sqlalchemy import Column, Integer
from sqlalchemy.orm import DeclarativeBase

from mind_castle.sqlalchemy_type import SecretData


class Base(DeclarativeBase):
    pass


class SimpleNoneModel(Base):
    __tablename__ = "testnonemodel"
    id = Column(Integer, primary_key=True)
    data = Column(SecretData("none"))


class SimpleMemoryModel(Base):
    __tablename__ = "testmemorymodel"
    id = Column(Integer, primary_key=True)
    data = Column(SecretData("memory"))


class SimpleJsonModel(Base):
    __tablename__ = "testjsonmodel"
    id = Column(Integer, primary_key=True)
    data = Column(SecretData("json"))


class SimpleAWSSecretsManagerModel(Base):
    __tablename__ = "testawssecretsmanagermodel"
    id = Column(Integer, primary_key=True)
    data = Column(SecretData("awssecretsmanager"))


class SimpleAWSKMSModel(Base):
    __tablename__ = "testawskmsmodel"
    id = Column(Integer, primary_key=True)
    data = Column(SecretData("awskms"))


class SimpleLocalEncryptionModel(Base):
    __tablename__ = "testlocalmodel"
    id = Column(Integer, primary_key=True)
    data = Column(SecretData("localencryption"))



import pytest
import base64
import os
from cryptography.hazmat.primitives.ciphers.aead import AESGCM

from mind_castle.stores.local_encryption import LocalEncryptionSecretStore
from mind_castle.secret_store_base import MindCastleSecret


@pytest.fixture(scope="module")
def aesgcm():
    key = base64.urlsafe_b64encode(os.urandom(16)).decode()
    os.environ["MIND_CASTLE_LOCAL_ENCRYPTION_SECRET"] = key
    return AESGCM(base64.urlsafe_b64decode(key))


def test_create_secret(aesgcm):
    secret_store = LocalEncryptionSecretStore()

    response = secret_store.create_secret("some_secret_value")

    blob = base64.urlsafe_b64decode(response.encrypted_value)
    nonce, ct = blob[:12], blob[12:]
    decrypted_value = aesgcm.decrypt(nonce, ct, None).decode()
    assert decrypted_value == "some_secret_value"
    assert response.mind_castle_secret_type == secret_store.store_type


def test_retrieve_secret(aesgcm):
    secret_store = LocalEncryptionSecretStore()
    secret = secret_store.create_secret("retrievable_secret")
    assert secret_store.retrieve_secret(secret) == "retrievable_secret"


def test_delete_secret(aesgcm):
    secret_store = LocalEncryptionSecretStore()
    secret_store.delete_secret(
        MindCastleSecret(
            mind_castle_secret_type=secret_store.store_type,
            encrypted_value="some_value",
        )
    )
    # Just shouldn't throw an error


def test_update_secret(aesgcm):
    secret_store = LocalEncryptionSecretStore()
    secret = secret_store.create_secret("updatable_secret")

    updated_secret = secret_store.update_secret(secret, "some_updated_value")

    blob = base64.urlsafe_b64decode(updated_secret.encrypted_value)
    nonce, ct = blob[:12], blob[12:]
    decrypted_value = aesgcm.decrypt(nonce, ct, None).decode()
    assert decrypted_value == "some_updated_value"
    assert updated_secret.mind_castle_secret_type == secret_store.store_type
